# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_admin_conf_vars', '0002_auto_20150413_2238'),
    ]

    operations = [
        migrations.AddField(
            model_name='configurationvariable',
            name='editable',
            field=models.BooleanField(default=True),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ConfigurationVariableConfig(AppConfig):
    name = 'django_admin_conf_vars'
    verbose_name = _(u'Configuration variables')

    def ready(self):
        from .global_vars import config
        config.load_attributes()



from django.contrib import admin
from .models import ConfigurationVariable


class ConfigurationVariableAdmin(admin.ModelAdmin):
    list_display = ('name', 'value', 'description', 'editable')
    fields = ('name', 'value', 'description', 'editable')

    def get_readonly_fields(self, request, obj=None):
        if obj:
            if not obj.editable:
                self.readonly_fields = ('value',)
            else:
                self.readonly_fields = ()
            return ('name', 'editable', 'description',) + self.readonly_fields
        return self.readonly_fields

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False


admin.site.register(ConfigurationVariable, ConfigurationVariableAdmin)



from django.conf import settings
from .models import ConfigurationVariable
import logging


VARS_MODULE_PATH_DEFAULT = 'global_vars'
logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger(__name__)


class VariablesManager(object):

    ATTRIBUTES = {}

    def __new__(cls, *a, **k):
        '''Constructor to make a Singleton class'''

        if not hasattr(cls, '_inst'):
            cls._inst = super(VariablesManager, cls).__new__(cls, *a, **k)
        return cls._inst

    def __getattr__(self, key):
        '''Give access to the values of the ATTRIBUTES dictionary
        eg: config.ATTRIBUTES['MY_VAR'] is equal to config.MY_VAR'''

        if not bool(len(self.ATTRIBUTES)):
            self.load_attributes()
        return self.ATTRIBUTES.get(key, None)

    def getAll(self):
        '''Return a dictionary with all variables'''

        if not bool(len(self.ATTRIBUTES)):
            self.load_attributes()
        return eval(str(self.ATTRIBUTES))

    def set(self, name, default=0, editable=True, description=""):
        '''Define a variable in DB and in memory'''

        var, created = ConfigurationVariable.objects.get_or_create(name=name)

        if created:
            var.value = default

        if not editable:
            var.value = default

        var.editable = editable
        var.description = description
        var.save(reload=False)

        # ATTRIBUTES is accesible by any instance of VariablesManager
        self.ATTRIBUTES[var.name] = var.value

    def reload(self, name, value):
        '''
            Update a variable in memory.
            ConfigurationVariable model use this method when a variable
            is updated via de django admin.
        '''

        self.ATTRIBUTES[name] = value

    def load_attributes(self):
        '''Read the variables from the VARS_MODULE_PATH'''

        try:
            vars_path = settings.VARS_MODULE_PATH
        except Exception:
            # logger.warning("*" * 55)
            logger.warning(
                " [WARNING] Using default VARS_MODULE_PATH = '{}'".format(
                    VARS_MODULE_PATH_DEFAULT))
            vars_path = VARS_MODULE_PATH_DEFAULT

        try:
            __import__(vars_path)
        except ImportError:
            logger.warning(" [WARNING] No module named '{}'".format(
                vars_path))
            logger.warning(" Please, read the docs: goo.gl/E82vkX\n".format(
                vars_path))


config = VariablesManager()



from django.db import models
from django.utils.translation import ugettext_lazy as _


class ConfigurationVariable(models.Model):
    name = models.CharField(max_length=100)
    value = models.CharField(max_length=255)
    editable = models.BooleanField(
        default=True,
        help_text=_(u'The variable can be edited in the admin or not'))
    description = models.TextField(
        default=True,
        blank=True,
        help_text=_(u'The description can only be edited in the code.'))

    def save(self, reload=True, *args, **kwargs):
        res = super(ConfigurationVariable, self).save(*args, **kwargs)
        if reload:
            from .global_vars import config
            config.reload(self.name, self.value)
        return res

    def __str__(self):
        return u"%s: %s" % (self.name, self.value)

    def __unicode__(self):
        return u"%s: %s" % (self.name, self.value)

    class Meta:
        verbose_name = _(u'Variable')
        verbose_name_plural = _(u'Variables list')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_admin_conf_vars', '0004_configurationvariable_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='configurationvariable',
            name='description',
            field=models.TextField(default=True, help_text='The description can only be edited in the code.', blank=True),
        ),
        migrations.AlterField(
            model_name='configurationvariable',
            name='editable',
            field=models.BooleanField(default=True, help_text='The variable can be edited in the admin or not'),
        ),
    ]
